@@ -69,11 +69,13 @@ enum edge_enum edge = EDGE_EAST;
 int edge_width=1;
 int restingx=-1;
 int restingy=-1;
-int emulate_wheel=1;
+int emulate_wheel=0;
+int emulate_nav=0;
 int wheel_button_up=4;
 int scroll_lines=1;
 int mac_mode=0;
 int hidden=0;
+extern int debug;
 
 int last_event_time = 0;
 
@@ -135,6 +137,7 @@ Bool CreateXWindow(void)
   XSizeHints wmHints;
   XGCValues gcv;
   int i;
+  int ew;
   
   Pixmap    nullPixmap;
   XColor    dummyColor;
@@ -159,9 +162,8 @@ Bool CreateXWindow(void)
  {
    int x,y;
    server_has_mitscreensaver=XScreenSaverQueryExtension(dpy, &x,&y);
-#ifdef DEBUG
-   fprintf(stderr,"MIT-SCREEN-SAVER = %d\n",server_has_mitscreensaver);
-#endif
+   if(debug)
+     fprintf(stderr,"MIT-SCREEN-SAVER = %d\n",server_has_mitscreensaver);
  }
 #endif
 
@@ -259,19 +261,20 @@ Bool CreateXWindow(void)
    }
  }
 #endif
-
-  topLevelWidth=edge_width;
-  topLevelHeight=edge_width;
+  ew=edge_width;
+  if(!ew) ew=1;
+  topLevelWidth=ew;
+  topLevelHeight=ew;
   wmHints.x=x_offset;
   wmHints.y=y_offset;
   
   switch(edge)
   {
-    case EDGE_EAST: wmHints.x=displayWidth-edge_width+x_offset;
+    case EDGE_EAST: wmHints.x=displayWidth-ew+x_offset;
     case EDGE_WEST: topLevelHeight=displayHeight;
       break;
       
-    case EDGE_SOUTH: wmHints.y=displayHeight-edge_width+y_offset;
+    case EDGE_SOUTH: wmHints.y=displayHeight-ew+y_offset;
     case EDGE_NORTH: topLevelWidth=displayWidth;
       break;
   }
@@ -334,7 +337,7 @@ Bool CreateXWindow(void)
 		    XA_ATOM,
 		    32,
 		    PropModeReplace,
-		    (char *)&t,
+		    (unsigned char *)&t,
 		    1);
   }
 #endif
@@ -348,8 +351,11 @@ Bool CreateXWindow(void)
   XSetWMProtocols(dpy, topLevel, &wmDeleteWindow, 1);
   
   XStoreName(dpy, topLevel, desktopName);
-  
-  XMapRaised(dpy, topLevel);
+
+  if(edge_width)
+    XMapRaised(dpy, topLevel);
+  else
+    hidden=1;
   
   /*
    * For multi-screen setups, we need to know if the mouse is on the 
@@ -502,15 +508,12 @@ Bool HandleXEvents(void)
   
   
   /* presumably XCheckMaskEvent is more efficient than XCheckIfEvent -GRM */
-    while (XCheckIfEvent(dpy, &ev, AllXEventsPredicate, NULL))
-/*  while (XCheckMaskEvent(dpy, ~0, &ev)) */
+  while(XCheckIfEvent(dpy, &ev, AllXEventsPredicate, NULL))
   {
     if (ev.xany.window == topLevel)
     {
-      
       if (!HandleTopLevelEvent(&ev))
 	return False;
-      
     }
     else if (ev.xany.window == DefaultRootWindow(dpy) ||
 	     ev.xany.window == RootWindow(dpy, 0)) {
@@ -566,10 +569,32 @@ static int sendpointerevent(int localx, int localy, int buttonmask)
   return SendPointerEvent(SCALEX(localx), SCALEY(localy), buttonmask);
 }
 
+
+void mapwindow(void)
+{
+  if(edge_width)
+  {
+    hidden=0;
+    XMapRaised(dpy, topLevel);
+  }
+}
+
+void hidewindow(void)
+{
+  hidden=1;
+  XUnmapWindow(dpy, topLevel);
+}
+
 static void grabit(int x, int y, int state)
 {
   Window selection_owner;
 
+  if(hidden)
+  {
+    XMapRaised(dpy, topLevel);
+    hidden=0;
+  }
+
   XGrabPointer(dpy, topLevel, True,
 	       PointerMotionMask | ButtonPressMask | ButtonReleaseMask,
 	       GrabModeAsync, GrabModeAsync,
@@ -605,6 +630,7 @@ static void grabit(int x, int y, int state)
 static void ungrabit(int x, int y, Window warpWindow)
 {
   int i;
+
   SendPointerEvent(restingx,restingy,0);
   if(x > -1 && y > -1 )
     XWarpPointer(dpy,None, warpWindow, 0,0,0,0, x_offset + x, y_offset + y);
@@ -621,14 +647,20 @@ static void ungrabit(int x, int y, Window warpWindow)
       modifierPressed[i]=False;
     }
   }
+
+  /* this is a workaround for some older vnc servers which don't
+   * send selection events unless you request a screen update
+   * It is possible that this should be an command-line option,
+   * but getting a one-pixel screen update should be reasonably
+   * harmless.
+   */
+  SendFramebufferUpdateRequest(0,0,1,1,0);
+
+  if(!edge_width) hidewindow();
   
   grabbed=0;
 }
 
-/*
- * HandleTopLevelEvent.
- */
-
 static void shortsleep(int usec)
 {
   struct timeval timeout;
@@ -638,6 +670,9 @@ static void shortsleep(int usec)
 }
 
 
+/*
+ * HandleTopLevelEvent.
+ */
 static Bool HandleTopLevelEvent(XEvent *ev)
 {
   Bool grab;
@@ -646,6 +681,10 @@ static Bool HandleTopLevelEvent(XEvent *ev)
   
   int buttonMask;
   KeySym ks;
+  static Atom COMPOUND_TEXT;
+  /* Atom: requestor asks for a list of supported targets (GRM 24 Oct 2003) */
+  static Atom TARGETS;
+
   
   SET_GRAB_TIMEOUT();
 
@@ -665,9 +704,41 @@ static Bool HandleTopLevelEvent(XEvent *ev)
       reply.xselection.time=req->time;
       reply.xselection.property=None;
       
+      if(COMPOUND_TEXT == None)
+	COMPOUND_TEXT = XInternAtom(dpy, "COMPOUND_TEXT", True);
+
+      /* Initialize TARGETS atom if required (GRM 24 Oct 2003) */
+      if(TARGETS == None) TARGETS = XInternAtom(dpy, "TARGETS", True);
+
       if(client_selection_text)
       {
-	if(req->target == XA_STRING)
+	if(req->target == TARGETS)
+	{
+          /*
+           * Requestor is asking what targets we support. Reply with what we
+           * support (XA_STRING and COMPOUND_TEXT).
+           *
+           * (GRM 24 Oct 2003)
+           */
+	  int ret;
+          int targets[2];
+	  if(req->property == None)
+	    req->property = TARGETS;
+
+          targets[0] = XA_STRING;
+          targets[1] = COMPOUND_TEXT;
+	  XChangeProperty(dpy,
+			  req->requestor,
+			  req->property,
+			  XA_INTEGER,
+			  8 * sizeof(int),
+			  PropModeReplace,
+                          (unsigned char *) targets,
+                          2);
+
+	  reply.xselection.property=req->property;
+        }
+	else if(req->target == XA_STRING || req->target == COMPOUND_TEXT)
 	{
 	  int ret;
 	  if(req->property == None)
@@ -689,6 +760,17 @@ static Bool HandleTopLevelEvent(XEvent *ev)
 	  reply.xselection.property=req->property;
 	  
 	}
+        else if(debug)
+        {
+            /*
+             * Requestor is asking for something we don't understand. Complain.
+             * (GRM 24 Oct 2003)
+             */
+            char *name;
+            name = XGetAtomName(dpy, req->target);
+            fprintf(stderr, "Unknown property target request %s\n", name);
+            XFree(name);
+        }
       }
       XSendEvent(dpy, req->requestor, 0,0, &reply);
       XFlush (dpy);
@@ -718,7 +800,7 @@ static Bool HandleTopLevelEvent(XEvent *ev)
 			     &data);
 
       if(format == 8)
-	SendClientCutText(data, len);
+	SendClientCutText((char *)data, len);
 #ifdef DEBUG
       fprintf(stderr,"GOT selection info: ret=%d type=%d fmt=%d len=%d bytes_left=%d %p '%s'\n",
 	ret, (int)t,format,len,bytes_left,data,data);
@@ -874,7 +956,9 @@ static Bool HandleTopLevelEvent(XEvent *ev)
       
     case ButtonPress:
     case ButtonRelease:
-      if (ev->xbutton.button > 3 && emulate_wheel)
+      if (emulate_wheel &&
+	  ev->xbutton.button >= 4 &&
+	  ev->xbutton.button <= 5)
       {
 	int l;
 	if (ev->xbutton.button == wheel_button_up)
@@ -895,6 +979,23 @@ static Bool HandleTopLevelEvent(XEvent *ev)
 	break;
       }
 
+      if (emulate_nav &&
+	  ev->xbutton.button >= 5 &&
+	  ev->xbutton.button <= 7)
+      {
+	int l, ctrlcode;
+	if (ev->xbutton.button == 6)
+	  ks = XK_Left;
+	else
+	  ks = XK_Right;
+
+	ctrlcode=XKeysymToKeycode(dpy, XK_Alt_L);
+	SendKeyEvent(XK_Alt_L, ev->type == ButtonPress);
+	modifierPressed[ctrlcode]=ev->type == ButtonPress;
+	SendKeyEvent(ks, ev->type == ButtonPress); /* keydown */
+	break;
+      }
+
       if(mac_mode && ev->xbutton.button == 3)
       {
 	int ctrlcode=XKeysymToKeycode(dpy, XK_Control_L);
@@ -951,7 +1052,10 @@ static Bool HandleTopLevelEvent(XEvent *ev)
 	  case XK_ISO_Left_Tab: ks=XK_Tab; break;
 	}
       }
-/*      fprintf(stderr,"-------- %x (%c) name=%s\n",ks,ks,keyname); */
+
+      if(debug)
+	fprintf(stderr,"  --> %x (%c) name=%s (%s)\n",ks,ks,keyname,
+		ev->type == KeyPress ? "down" : "up");
 
       /* We assume that typing means an unlocked display */
       remote_is_locked=0;
@@ -1000,18 +1104,6 @@ int get_root_int_prop(Atom property)
   return ret;
 }
 
-void mapwindow(void)
-{
-  hidden=0;
-  XMapRaised(dpy, topLevel);
-}
-
-void hidewindow(void)
-{
-  hidden=1;
-  XUnmapWindow(dpy, topLevel);
-}
-
 void check_desktop(void)
 {
   int t=requested_desktop;
